@font-face {
  font-family: fira-sans;
  src: url("/app/share/fonts/FiraSans-Regular.ttf") format("ttf"),
       local("FiraSans-Regular"),
       url("https://fonts.gstatic.com/s/firasans/v10/va9E4kDNxMZdWfMOD5Vvl4jL.woff2") format("woff2");
}

@font-face {
  font-family: fira-mono;
  src: url("/app/share/fonts/FiraMono-Regular.ttf") format("ttf"),
       local("FiraMono-Regular"),
       url("https://fonts.gstatic.com/s/firamono/v9/N0bX2SlFPv1weGeLZDtgJv7S.woff2") format("woff2");
}

@font-face {
  font-family: color-emoji;
  src: local("Noto Color Emoji"), local("Apple Color Emoji"), local("Segoe UI Emoji"), local("Segoe UI Symbol");
}

:root {
  /* This is GitHub's default color scheme, which should be overridden per theme. */
  --text-color: #24292e;
  --background-color: #ffffff;
  --alt-background-color: #f6f8fa;
  --link-color: #0366d6;
  --blockquote-text-color: #6a737d;
  --blockquote-border-color: #dfe2e5;
  --header-border-color: #eaecef;
  --hr-background-color: #e1e4e8;
  --table-tr-border-color: #c6cbd1;
  --table-td-border-color: #dfe2e5;
  --kbd-text-color: #444d56;
  --kbd-background-color: #fafbfc;
  --kbd-border-color: #c6cbd1;
  --kbd-shadow-color: #959da5;
}

* {
  box-sizing: border-box;
}

html {
  font-size: 16px;
}

body {
  color: var(--text-color);
  background-color: var(--background-color);
  font-family: "Fira Sans", fira-sans, sans-serif, color-emoji;
  line-height: 1.5;
  word-wrap: break-word;
  max-width: 980px;
  margin: auto;
  margin-top: 60px;
  padding: 4em;
}

@media screen and (max-width: 799px) {
  html {
    font-size: 14px;
  }

  body {
    padding: 1em;
  }
}

@media screen and (min-width: 1280px) {
  html {
    font-size: 18px;
  }
}

a {
  background-color: transparent;
  color: var(--link-color);
  text-decoration: none;
}

a:active,
a:hover {
  outline-width: 0;
}

a:hover {
  text-decoration: underline;
}

strong {
  font-weight: 600;
}

img {
  border-style: none;
}

hr {
  box-sizing: content-box;
  height: 0.25em;
  padding: 0;
  margin: 1.5em 0;
  overflow: hidden;
  background-color: var(--hr-background-color);
  border: 0;
}

hr::before {
  display: table;
  content: "";
}

hr::after {
  display: table;
  clear: both;
  content: "";
}

input {
  font-family: inherit;
  font-size: inherit;
  line-height: inherit;
  margin: 0;
  overflow: visible;
}

[type="checkbox"] {
  box-sizing: border-box;
  padding: 0;
}

table {
  border-spacing: 0;
  border-collapse: collapse;
}

td,
th {
  padding: 0;
}

h1,
h2,
h3,
h4,
h5,
h6 {
  font-weight: 600;
  margin: 0;
}

h1 {
  font-size: 2em;
}

h2 {
  font-size: 1.5em;
}

h3 {
  font-size: 1.25em;
}

h4 {
  font-size: 1em;
}

h5 {
  font-size: 0.875em;
}

h6 {
  font-size: 0.85em;
}

p {
  margin-top: 0;
  margin-bottom: 0.625em;
}

blockquote {
  margin: 0;
}

ul,
ol {
  padding-left: 0;
  margin-top: 0;
  margin-bottom: 0;
}

ol ol,
ul ol {
  list-style-type: lower-roman;
}

ul ul ol,
ul ol ol,
ol ul ol,
ol ol ol {
  list-style-type: lower-alpha;
}

dd {
  margin-left: 0;
}

code,
kbd,
pre {
  font-family: "Fira Mono", fira-mono, monospace, color-emoji;
  font-size: 1em;
  word-wrap: normal;
}

code {
  border-radius: 0.1875em;
  font-size: 0.85em;
  padding: 0.2em 0.4em;
  margin: 0;
}

pre {
  margin-top: 0;
  margin-bottom: 0;
  font-size: 0.75em;
}

pre>code {
  padding: 0;
  margin: 0;
  font-size: 1em;
  word-break: normal;
  white-space: pre;
  background: transparent;
  border: 0;
}

.highlight {
  margin-bottom: 1em;
}

.highlight pre {
  margin-bottom: 0;
  word-break: normal;
}

.highlight pre,
pre {
  padding: 1em;
  overflow: auto;
  font-size: 0.85em;
  line-height: 1.5;
  background-color: var(--alt-background-color);
  border-radius: 0.1875em;
}

pre code {
  background-color: transparent;
  border: 0;
  display: inline;
  padding: 0;
  margin: 0;
  overflow: visible;
  line-height: inherit;
  word-wrap: normal;
}

.pl-0 {
  padding-left: 0 !important;
}

.pl-1 {
  padding-left: 0.25em !important;
}

.pl-2 {
  padding-left: 0.5em !important;
}

.pl-3 {
  padding-left: 1em !important;
}

.pl-4 {
  padding-left: 1.5em !important;
}

.pl-5 {
  padding-left: 2em !important;
}

.pl-6 {
  padding-left: 2.5em !important;
}

.markdown-body::before {
  display: table;
  content: "";
}

.markdown-body::after {
  display: table;
  clear: both;
  content: "";
}

.markdown-body>*:first-child {
  margin-top: 0 !important;
}

.markdown-body>*:last-child {
  margin-bottom: 0 !important;
}

a:not([href]) {
  color: inherit;
  text-decoration: none;
}

.anchor {
  float: left;
  padding-right: 0.25em;
  margin-left: -1.25em;
  line-height: 1;
}

.anchor:focus {
  outline: none;
}

p,
blockquote,
ul,
ol,
dl,
table,
pre {
  margin-top: 0;
  margin-bottom: 1em;
}

blockquote {
  padding: 0 1em;
  color: var(--blockquote-text-color);
  border-left: 0.25em solid var(--blockquote-border-color);
}

blockquote>:first-child {
  margin-top: 0;
}

blockquote>:last-child {
  margin-bottom: 0;
}

kbd {
  display: inline-block;
  padding: 0.1875em 0.3125em;
  font-size: 0.6875em;
  line-height: 1;
  color: var(--kbd-text-color);
  vertical-align: middle;
  background-color: var(--kbd-background-color);
  border: solid 1px var(--kbd-border-color);
  border-bottom-color: var(--kbd-shadow-color);
  border-radius: 3px;
  box-shadow: inset 0 -1px 0 var(--kbd-shadow-color);;
}

h1,
h2,
h3,
h4,
h5,
h6 {
  margin-top: 1.5em;
  margin-bottom: 1em;
  font-weight: 600;
  line-height: 1.25;
}

h1:hover .anchor,
h2:hover .anchor,
h3:hover .anchor,
h4:hover .anchor,
h5:hover .anchor,
h6:hover .anchor {
  text-decoration: none;
}

h1 {
  padding-bottom: 0.3em;
  font-size: 2em;
  border-bottom: 1px solid var(--header-border-color);
}

h2 {
  padding-bottom: 0.3em;
  font-size: 1.5em;
  border-bottom: 1px solid var(--header-border-color);
}

h3 {
  font-size: 1.25em;
}

h4 {
  font-size: 1em;
}

h5 {
  font-size: 0.875em;
}

h6 {
  font-size: 0.85em;
  opacity: 0.67;
}

ul,
ol {
  padding-left: 2em;
}

ul ul,
ul ol,
ol ol,
ol ul {
  margin-top: 0;
  margin-bottom: 0;
}

li {
  overflow-wrap: break-word;
}

li>p {
  margin-top: 1em;
}

li+li {
  margin-top: 0.25em;
}

dl {
  padding: 0;
}

dl dt {
  padding: 0;
  margin-top: 1em;
  font-size: 1em;
  font-style: italic;
  font-weight: 600;
}

dl dd {
  padding: 0 1em;
  margin-bottom: 1em;
}

table {
  display: block;
  width: 100%;
  overflow: auto;
}

table th {
  font-weight: 600;
}

table th,
table td {
  padding: 0.375em 0.8125em;
  border: 1px solid var(--table-td-border-color);
}

table tr {
  background-color: var(--background-color);
  border-top: 1px solid var(--table-tr-border-color);
}

table tr:nth-child(2n) {
  background-color: var(--alt-background-color);
}

img {
  max-width: 100%;
  box-sizing: content-box;
}

img[align=right] {
  padding-left: 1.25em;
}

img[align=left] {
  padding-right: 1.25em;
}

.task-list-item {
  list-style-type: none;
}

.task-list-item+.task-list-item {
  margin-top: 0.1875em;
}

.task-list-item input {
  margin: 0 0.2em 0.25em -1.6em;
  vertical-align: middle;
}
